@@ -45,17 +45,16 @@ rb_file (const char *filename)
 	char *ret;
 	int i;
 
-	static char *paths[] =
-	{
+	static char *paths[] = {
+		SHARE_UNINSTALLED_DIR "/",
+		SHARE_UNINSTALLED_DIR "/glade/",
+		SHARE_UNINSTALLED_DIR "/art/",
+		SHARE_UNINSTALLED_DIR "/node-views/"
 		SHARE_DIR "/",
 		SHARE_DIR "/glade/",
 		SHARE_DIR "/art/",
 		SHARE_DIR "/views/",
 		SHARE_DIR "/node-views/",
-		SHARE_UNINSTALLED_DIR "/",
-		SHARE_UNINSTALLED_DIR "/glade/",
-		SHARE_UNINSTALLED_DIR "/art/",
-		SHARE_UNINSTALLED_DIR "/node-views/"
 	};
 	
 	g_assert (files != NULL);
@@ -64,11 +63,9 @@ rb_file (const char *filename)
 	if (ret != NULL)
 		return ret;
 
-	for (i = 0; i < (int) G_N_ELEMENTS (paths); i++)
-	{
+	for (i = 0; i < (int) G_N_ELEMENTS (paths); i++) {
 		ret = g_strconcat (paths[i], filename, NULL);
-		if (g_file_test (ret, G_FILE_TEST_EXISTS) == TRUE)
-		{
+		if (g_file_test (ret, G_FILE_TEST_EXISTS) == TRUE) {
 			g_hash_table_insert (files, g_strdup (filename), ret);
 			return (const char *) ret;
 		}
@@ -674,3 +671,28 @@ rb_uri_handle_recursively (const char *text_uri,
 	gnome_vfs_file_info_list_free (list);
 	gnome_vfs_uri_unref (uri);
 }
+
+
+GnomeVFSResult
+rb_uri_mkstemp (const char *prefix, char **uri_ret, GnomeVFSHandle **ret)
+{
+	GnomeVFSHandle *handle = NULL;
+	char *uri = NULL;
+	GnomeVFSResult result = GNOME_VFS_ERROR_FILE_EXISTS;
+	
+	
+	do {
+		g_free (uri);
+		uri = g_strdup_printf ("%s%06X", prefix, g_random_int_range (0, 0xFFFFFF));
+		result = gnome_vfs_create (&handle, uri, GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_RANDOM, TRUE,  0644);
+	} while (result == GNOME_VFS_ERROR_FILE_EXISTS);
+
+	if (result == GNOME_VFS_OK) {
+		*uri_ret = uri;
+		*ret = handle;
+	} else {
+		g_free (uri);
+	}
+	return result;
+}
+
