@@ -53,9 +53,9 @@ static struct
 	const char *mimetype;
 	const char *plugin;
 } rb_metadata_type_map[] = {
-#if 0
 	{"application/x-id3", "id3tag"},
-#endif
+	{"application/ogg", NULL},
+	{"audio/x-flac", "flactag"},
 };
 
 static GObjectClass *parent_class = NULL;
@@ -152,6 +152,8 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_COMMENT;
 	else if (!strcmp (tag, GST_TAG_TRACK_NUMBER))
 		return RB_METADATA_FIELD_TRACK_NUMBER;
+	else if (!strcmp (tag, GST_TAG_ALBUM_VOLUME_NUMBER))
+		return RB_METADATA_FIELD_DISC_NUMBER;
 	else if (!strcmp (tag, GST_TAG_DESCRIPTION))
 		return RB_METADATA_FIELD_DESCRIPTION;
 	else if (!strcmp (tag, GST_TAG_VERSION))
@@ -205,6 +207,8 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 		return GST_TAG_COMMENT;
 	case RB_METADATA_FIELD_TRACK_NUMBER:
 		return GST_TAG_TRACK_NUMBER;
+	case RB_METADATA_FIELD_DISC_NUMBER:
+		return GST_TAG_ALBUM_VOLUME_NUMBER;
 	case RB_METADATA_FIELD_DESCRIPTION:
 		return GST_TAG_DESCRIPTION;
 	case RB_METADATA_FIELD_VERSION:
@@ -286,12 +290,40 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	val = gst_tag_list_get_value_index (list, tag, 0);
 	newval = g_new0 (GValue, 1);
 	g_value_init (newval, rb_metadata_get_field_type (md, field));
-	if (g_value_transform (val, newval))
-		g_hash_table_insert (md->priv->metadata, GINT_TO_POINTER (field),
-				     newval);
-	else
+	if (!g_value_transform (val, newval)) {
+		
 		rb_debug ("Could not transform tag value type %s into %s",
-			  g_type_name (G_VALUE_TYPE (val)), g_type_name (G_VALUE_TYPE (newval)));
+			  g_type_name (G_VALUE_TYPE (val)), 
+			  g_type_name (G_VALUE_TYPE (newval)));
+		return;
+	}
+
+	switch (field) {
+	case RB_METADATA_FIELD_BITRATE: {
+		/* GStreamer sends us bitrate in bps, but we need it in kbps*/
+		gint bitrate;
+		bitrate = g_value_get_int (newval);
+		g_value_set_int (newval, bitrate/1000);		
+		break;
+	}
+
+	case RB_METADATA_FIELD_DURATION: {
+		/* GStreamer sends us duration in ns, 
+		 * but we need it in seconds
+		 */
+		guint64 duration;
+		duration = g_value_get_uint64 (val);
+		g_value_set_long (newval, duration/(1000*1000*1000));
+		break;
+	}
+
+	default:
+		break;
+	}
+
+	g_hash_table_insert (md->priv->metadata, 
+			     GINT_TO_POINTER (field),
+			     newval);
 }
 
 static void
@@ -408,26 +440,45 @@ rb_metadata_load (RBMetaData *md,
 		/* We caught the first buffer, which means the decoder should have read all
 		 * of the metadata, and should know the length now.
 		 */
-		GstFormat format = GST_FORMAT_TIME;
-		gint64 length;
-		
-		if (gst_element_query (md->priv->sink, GST_QUERY_TOTAL, &format, &length)) {
-			GValue *newval = g_new0 (GValue, 1);
-
-			rb_debug ("duration query succeeded");
+		if (g_hash_table_lookup (md->priv->metadata, GINT_TO_POINTER (RB_METADATA_FIELD_DURATION)) == NULL) {
+			GstFormat format = GST_FORMAT_TIME;
+			gint64 length;
 			
-			g_value_init (newval, G_TYPE_LONG);
-			/* FIXME - use guint64 for duration? */
-			g_value_set_long (newval, (long) (length / (1 * 1000 * 1000 * 1000)));
-			g_hash_table_insert (md->priv->metadata, GINT_TO_POINTER (RB_METADATA_FIELD_DURATION),
-					     newval);
-		} else {
-			rb_debug ("duration query failed!");
+			if (gst_element_query (md->priv->sink, GST_QUERY_TOTAL,
+					       &format, &length)) {
+				GValue *newval = g_new0 (GValue, 1);
+				
+				rb_debug ("duration query succeeded");
+				
+				g_value_init (newval, G_TYPE_LONG);
+				/* FIXME - use guint64 for duration? */
+				g_value_set_long (newval, (long) (length / (1 * 1000 * 1000 * 1000)));
+				g_hash_table_insert (md->priv->metadata, GINT_TO_POINTER (RB_METADATA_FIELD_DURATION),
+						     newval);
+			} else {
+				rb_debug ("duration query failed!");
+			}
 		}
 	} else if (md->priv->eos)
 		rb_debug ("caught eos without handoff!");
 	
 	gst_element_set_state (pipeline, GST_STATE_NULL);
+
+	if (!md->priv->type) {
+		rb_debug ("couldn't get MIME type for %s", uri);
+		goto out;
+	}
+
+	/* FIXME
+	 * For now, we simply ignore files with an unknown MIME
+	 * type. This will be fixed once GStreamer gives us
+	 * a good way to detect audio. */
+	if (!rb_metadata_can_save (md, md->priv->type)) {
+		rb_debug ("ignoring file %s with detected type %s",
+			  uri, md->priv->type);
+		goto out;
+	}
+	
 	if (md->priv->error) {
 		g_propagate_error (error, md->priv->error);
 	}
@@ -447,10 +498,14 @@ rb_metadata_load (RBMetaData *md,
 gboolean
 rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
+	/* Disabled until we can save reliably.
+	 */
+#if 0
 	int i;
 	for (i = 0; i < G_N_ELEMENTS (rb_metadata_type_map); i++)
 		if (!strcmp (rb_metadata_type_map[i].mimetype, mimetype))
 			return TRUE;
+#endif
 	return FALSE;
 }
 
