@@ -32,6 +32,7 @@
 #include "rb-thread-helpers.h"
 #include "gsequence.h"
 #include "rb-tree-dnd.h"
+#include "rb-marshal.h"
 
 static void rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass);
 static void rhythmdb_query_model_tree_model_init (GtkTreeModelIface *iface);
@@ -47,16 +48,15 @@ static void rhythmdb_query_model_get_property (GObject *object,
 					       guint prop_id,
 					       GValue *value,
 					       GParamSpec *pspec);
-static GSequencePtr rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
-						    RhythmDBEntry *entry,
-						    gboolean lock);
+static void rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
+					    RhythmDBEntry *entry);
 static void rhythmdb_query_model_do_delete (RhythmDBQueryModel *model,
-					    RhythmDBEntry *entry,
-					    gboolean lock);
+					    RhythmDBEntry *entry);
 static void rhythmdb_query_model_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 						 RhythmDBQueryModel *model);
 static void rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-						   RhythmDBQueryModel *model);
+						   RhythmDBPropType prop, const GValue *old,
+						   const GValue *new, RhythmDBQueryModel *model);
 static void rhythmdb_query_model_entry_deleted_cb (RhythmDB *db, RhythmDBEntry *entry,
 						   RhythmDBQueryModel *model);
 
@@ -75,6 +75,11 @@ static gboolean rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_des
 							GtkTreePath *dest,
 							GtkTreeViewDropPosition pos,
 							GtkSelectionData  *selection_data);
+static gboolean rhythmdb_query_model_row_drop_position (RbTreeDragDest   *drag_dest,
+							GtkTreePath       *dest_path,
+							GList *targets,
+							GtkTreeViewDropPosition *pos);
+
 static GtkTreeModelFlags rhythmdb_query_model_get_flags (GtkTreeModel *model);
 static gint rhythmdb_query_model_get_n_columns (GtkTreeModel *tree_model);
 static GType rhythmdb_query_model_get_column_type (GtkTreeModel *tree_model, int index);
@@ -101,6 +106,7 @@ static gboolean rhythmdb_query_model_iter_parent (GtkTreeModel *tree_model,
 						  GtkTreeIter  *child);
 static GSequencePtr choose_sequence_element (GSequence *seq);
 
+static gboolean idle_poll_model (RhythmDBQueryModel *model);
 
 static const GtkTargetEntry rhythmdb_query_model_drag_types[] = { { "text/uri-list", 0, 0 },};
 
@@ -109,12 +115,18 @@ static GtkTargetList *rhythmdb_query_model_drag_target_list = NULL;
 struct RhythmDBQueryModelUpdate
 {
 	enum {
-		RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED,
+		RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED,
 		RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED,
 		RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED,
 		RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE,
 	} type;
 	RhythmDBEntry *entry;
+	GPtrArray *entries;
+
+	/* Only used for _ROW_CHANGED */
+	RhythmDBPropType prop;
+	GValue old;
+	GValue new;
 };
 
 struct RhythmDBQueryModelPrivate
@@ -135,6 +147,8 @@ struct RhythmDBQueryModelPrivate
 	
 	gboolean connected;
 
+	guint model_poll_id;
+
 	glong total_duration;
 	GnomeVFSFileSize total_size;
 
@@ -163,6 +177,7 @@ enum
 enum
 {
 	COMPLETE,
+	ENTRY_PROP_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -291,6 +306,15 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 							   0, G_MAXINT, 0,
 							   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	rhythmdb_query_model_signals[ENTRY_PROP_CHANGED] =
+		g_signal_new ("entry-prop-changed",
+			      RHYTHMDB_TYPE_QUERY_MODEL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, entry_prop_changed),
+			      NULL, NULL,
+			      rb_marshal_VOID__POINTER_INT_POINTER_POINTER,
+			      G_TYPE_NONE, 4, G_TYPE_POINTER,
+			      G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
 	rhythmdb_query_model_signals[COMPLETE] =
 		g_signal_new ("complete",
 			      RHYTHMDB_TYPE_QUERY_MODEL,
@@ -331,7 +355,7 @@ rhythmdb_query_model_drag_dest_init (RbTreeDragDestIface *iface)
 {
 	iface->drag_data_received = rhythmdb_query_model_drag_data_received;
 	iface->row_drop_possible = rhythmdb_query_model_row_drop_possible;
-	iface->row_drop_position = NULL;
+	iface->row_drop_position = rhythmdb_query_model_row_drop_position;
 }
 
 static void
@@ -434,12 +458,44 @@ rhythmdb_query_model_init (RhythmDBQueryModel *model)
 	model->priv->pending_updates = g_async_queue_new ();
 
 	model->priv->reorder_drag_and_drop = FALSE;
+
+	model->priv->model_poll_id = g_idle_add ((GSourceFunc) idle_poll_model, model);
+}
+
+static void
+rhythmdb_query_model_free_pending_update (RhythmDBQueryModel *model,
+					  struct RhythmDBQueryModelUpdate *update)
+{
+	switch (update->type) {
+	case RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED:
+	{
+		guint i;
+		for (i = 0; i < update->entries->len; i++)
+			rhythmdb_entry_unref (model->priv->db,
+					      g_ptr_array_index (update->entries, i));
+		break;
+		g_ptr_array_free (update->entries, TRUE);
+	}
+	case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED:
+		rhythmdb_entry_unref (model->priv->db, update->entry);
+		g_value_unset (&update->old);
+		g_value_unset (&update->new);
+		break;
+	case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED:
+		rhythmdb_entry_unref (model->priv->db, update->entry);
+		break;
+	case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
+		break;
+	}
+
+	g_free (update);
 }
 
 static void
 rhythmdb_query_model_finalize (GObject *object)
 {
 	RhythmDBQueryModel *model;
+	struct RhythmDBQueryModelUpdate *update;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RHYTHMDB_IS_QUERY_MODEL (object));
@@ -450,18 +506,16 @@ rhythmdb_query_model_finalize (GObject *object)
 
 	rb_debug ("finalizing query model");
 
+	g_source_remove (model->priv->model_poll_id);
+
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->entries);
 
 	if (model->priv->query)
 		rhythmdb_query_free (model->priv->query);
 
-	{
-		struct RhythmDBQueryModelUpdate *update;
-		while ((update = g_async_queue_try_pop (model->priv->pending_updates)) != NULL) {
-			g_free (update);
-		}
-	}
+	while ((update = g_async_queue_try_pop (model->priv->pending_updates)) != NULL)
+		rhythmdb_query_model_free_pending_update (model, update);
 
 	g_async_queue_unref (model->priv->query_complete);
 	g_async_queue_unref (model->priv->pending_updates);
@@ -556,16 +610,32 @@ rhythmdb_query_model_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 
 static void
 rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-				       RhythmDBQueryModel *model)
+				       RhythmDBPropType prop, const GValue *old,
+				       const GValue *new, RhythmDBQueryModel *model)
 {
+	if (!model->priv->connected) {
+		return;
+	}
+
 	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL) {
 		struct RhythmDBQueryModelUpdate *update;
 
+		if (model->priv->query &&
+		    !rhythmdb_evaluate_query (db, model->priv->query, entry)) {
+			rhythmdb_query_model_remove_entry (model, entry);
+			return;
+		}
+
 		rb_debug ("queueing entry change");
 
-		update = g_new (struct RhythmDBQueryModelUpdate, 1);
+		update = g_new0 (struct RhythmDBQueryModelUpdate, 1);
 		update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED;
 		update->entry = entry;
+		update->prop = prop;
+		g_value_init (&update->old, G_VALUE_TYPE (old));
+		g_value_copy (old, &update->old);
+		g_value_init (&update->new, G_VALUE_TYPE (new));
+		g_value_copy (new, &update->new);
 	
 		/* Called with a locked database */
 		rhythmdb_entry_ref_unlocked (model->priv->db, entry);
@@ -587,53 +657,53 @@ rhythmdb_query_model_entry_deleted_cb (RhythmDB *db, RhythmDBEntry *entry,
 /* Threading: Called from the database context, holding a db write lock
  */
 void
-rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+rhythmdb_query_model_add_entries (RhythmDBQueryModel *model, GPtrArray *entries)
 {
 	struct RhythmDBQueryModelUpdate *update;
+	guint i;
 
 	if (model->priv->max_count > 0
 	    && g_hash_table_size (model->priv->reverse_map) >= model->priv->max_count)
 		return;
 	/* Check size later */
 
-	if (!model->priv->connected) {
-		rhythmdb_query_model_do_insert (model, entry, FALSE);
-		return;
-	}
-
 	update = g_new (struct RhythmDBQueryModelUpdate, 1);
-	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED;
-	update->entry = entry;
+	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED;
+	update->entries = entries;
 
 	/* Called with a locked database */
-	rhythmdb_entry_ref_unlocked (model->priv->db, entry);
+	for (i = 0; i < update->entries->len; i++)
+		rhythmdb_entry_ref_unlocked (model->priv->db,
+					     g_ptr_array_index (update->entries, i));
 
 	g_async_queue_push (model->priv->pending_updates, update);
 }
 
+void
+rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+{
+	GPtrArray *entries = g_ptr_array_new ();
+
+	g_ptr_array_add (entries, entry);
+	rhythmdb_query_model_add_entries (model, entries);
+}
+
 
 /* Threading: Called from the database context, holding a db write lock
  */
 void
 rhythmdb_query_model_remove_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry)
 {
-	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL) {
-		struct RhythmDBQueryModelUpdate *update;
-
-		if (!model->priv->connected) {
-			rhythmdb_query_model_do_delete (model, entry, FALSE);
-			return;
-		}
-
-		update = g_new (struct RhythmDBQueryModelUpdate, 1);
-		update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED;
-		update->entry = entry;
-
-		/* Called with a locked database */
-		rhythmdb_entry_ref_unlocked (model->priv->db, entry);
-
-		g_async_queue_push (model->priv->pending_updates, update);
-	}
+	struct RhythmDBQueryModelUpdate *update;
+	
+	update = g_new (struct RhythmDBQueryModelUpdate, 1);
+	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED;
+	update->entry = entry;
+	
+	/* Called with a locked database */
+	rhythmdb_entry_ref_unlocked (model->priv->db, entry);
+	
+	g_async_queue_push (model->priv->pending_updates, update);
 }
 
 GnomeVFSFileSize
@@ -663,44 +733,43 @@ compare_times (GTimeVal *a, GTimeVal *b)
 		return -1;
 }
 
-static GSequencePtr
+static void
 rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
-				RhythmDBEntry *entry,
-				gboolean lock)
+				RhythmDBEntry *entry)
 {
 	GSequencePtr ptr;
-	long size;
+	guint64 size;
 	long duration;
+	GtkTreePath *path;
+	GtkTreeIter iter;
 
 	/* we check again if the entry already exists in the hash table */
 	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL)
-		return NULL;
+		return;
 
 	if (model->priv->max_count > 0
 	    && g_hash_table_size (model->priv->reverse_map) >= model->priv->max_count)
-		return NULL;
+		return;
 
-	if (lock)
-		rhythmdb_read_lock (model->priv->db);
-	size = rhythmdb_entry_get_long (model->priv->db, entry,
-					RHYTHMDB_PROP_FILE_SIZE);
+	rhythmdb_read_lock (model->priv->db);
+	size = rhythmdb_entry_get_uint64 (model->priv->db, entry,
+					  RHYTHMDB_PROP_FILE_SIZE);
 	duration = rhythmdb_entry_get_long (model->priv->db,
 					    entry, RHYTHMDB_PROP_DURATION);
-	if (lock)
-		rhythmdb_read_unlock (model->priv->db);
+	rhythmdb_read_unlock (model->priv->db);
 
 	if (model->priv->max_size_mb > 0
 	    && ((model->priv->total_size + size) / (1024*1024)) >= model->priv->max_size_mb)
-		return NULL;
+		return;
+
+	rhythmdb_entry_ref (model->priv->db, entry);
 
 	if (model->priv->sort_func) {
-		if (lock)
-			rhythmdb_read_lock (model->priv->db);
+		rhythmdb_read_lock (model->priv->db);
 		ptr = g_sequence_insert_sorted (model->priv->entries, entry,
 						model->priv->sort_func,
 						model->priv->sort_user_data);
-		if (lock)
-			rhythmdb_read_unlock (model->priv->db);
+		rhythmdb_read_unlock (model->priv->db);
 	} else {
 		ptr = g_sequence_get_end_ptr (model->priv->entries);
 		g_sequence_insert (ptr, entry);
@@ -711,33 +780,73 @@ rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
 
 	model->priv->total_duration += duration;
 	model->priv->total_size += size;
-
-	return ptr;
+	
+	iter.stamp = model->priv->stamp;
+	iter.user_data = ptr;
+	path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
+					      &iter);
+	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model),
+				     path, &iter);
+	gtk_tree_path_free (path);
 }
 
 static void
 rhythmdb_query_model_do_delete (RhythmDBQueryModel *model,
-				RhythmDBEntry *entry,
-				gboolean lock)
+				RhythmDBEntry *entry)
 {
+	GtkTreePath *path;
 	GSequencePtr ptr;
-	
+	int index;
+
 	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 
 	if (ptr == NULL)
 		return;
 
-	if (lock)
-		rhythmdb_read_lock (model->priv->db);
+	index = g_sequence_ptr_get_position (ptr);
+	
+	path = gtk_tree_path_new ();
+	gtk_tree_path_append_index (path, index);
+	rb_debug ("emitting row deleted");
+	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
+	gtk_tree_path_free (path);
+	
+	rhythmdb_read_lock (model->priv->db);
 	model->priv->total_duration -= rhythmdb_entry_get_long (model->priv->db, entry,
 								RHYTHMDB_PROP_DURATION);
-	model->priv->total_size -= rhythmdb_entry_get_long (model->priv->db, entry,
-							    RHYTHMDB_PROP_FILE_SIZE);
-	if (lock)
-		rhythmdb_read_unlock (model->priv->db);
+	model->priv->total_size -= rhythmdb_entry_get_uint64 (model->priv->db, entry,
+							      RHYTHMDB_PROP_FILE_SIZE);
+	rhythmdb_read_unlock (model->priv->db);
 
 	g_sequence_remove (ptr);
 	g_hash_table_remove (model->priv->reverse_map, entry);
+	
+	rhythmdb_entry_unref (model->priv->db, entry);
+}
+
+static gboolean
+idle_poll_model (RhythmDBQueryModel *model)
+{
+	gboolean did_sync;
+	GTimeVal timeout;
+
+	g_get_current_time (&timeout);
+	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
+
+	GDK_THREADS_ENTER ();
+
+	did_sync = rhythmdb_query_model_poll (model, &timeout);
+
+	if (did_sync)
+		model->priv->model_poll_id =
+			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) idle_poll_model, model, NULL);
+	else
+		model->priv->model_poll_id =
+			g_timeout_add (300, (GSourceFunc) idle_poll_model, model);
+
+	GDK_THREADS_LEAVE ();
+
+	return FALSE;
 }
 	
 /* Threading: main thread only, should hold GDK lock
@@ -757,7 +866,6 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 		GtkTreePath *path;
 		GtkTreeIter iter;
 		GSequencePtr ptr;
-		int index;
 
 		if (update == NULL)
 			break;
@@ -765,19 +873,12 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 		iter.stamp = model->priv->stamp;
 
 		switch (update->type) {
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED:
+		case RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED:
 		{
-			ptr = rhythmdb_query_model_do_insert (model, update->entry, TRUE);
-
-			if (ptr == NULL)
-				break;
-
-			iter.user_data = ptr;
-			path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
-							      &iter);
-			gtk_tree_model_row_inserted (GTK_TREE_MODEL (model),
-						     path, &iter);
-			gtk_tree_path_free (path);
+			guint i;
+			for (i = 0; i < update->entries->len; i++)
+				rhythmdb_query_model_do_insert (model, g_ptr_array_index (update->entries, i));
+			break;
 		}
 		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED:
 		{
@@ -787,6 +888,10 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 			if (ptr == NULL)
 				break;
 
+			g_signal_emit (G_OBJECT (model),
+				       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
+				       update->entry, update->prop, &update->old, &update->new);
+
 			iter.user_data = ptr;
 			path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
 							      &iter);
@@ -798,21 +903,8 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 		}
 		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED:
 		{
-			ptr = g_hash_table_lookup (model->priv->reverse_map,
-						   update->entry);
-
-			if (ptr == NULL)
-				break;
-			index = g_sequence_ptr_get_position (ptr);
-
-			path = gtk_tree_path_new ();
-			gtk_tree_path_append_index (path, index);
-			rb_debug ("emitting row deleted");
-			gtk_tree_model_row_deleted (GTK_TREE_MODEL (model),
-						    path);
-			rhythmdb_query_model_do_delete (model, update->entry, TRUE);
-			
-			gtk_tree_path_free (path);
+			rhythmdb_query_model_do_delete (model, update->entry);
+			break;
 		}
 		case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
 		{
@@ -833,29 +925,8 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 		}
 	}
 	
-	for (tem = processed; tem; tem = tem->next) {
-		struct RhythmDBQueryModelUpdate *update = tem->data;
-			
-		switch (update->type) {
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED:
-			/* We steal the ref here from the update queue */
-			break;
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED:
-			rhythmdb_entry_unref (model->priv->db, update->entry);
-			break;
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED:
-			/* Unref twice; once because we were holding a ref originally
-			   due to storing it, and another time because we had it
-			   in the update queue. */
-			rhythmdb_entry_unref (model->priv->db, update->entry);
-			rhythmdb_entry_unref (model->priv->db, update->entry);
-			break;
-		case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
-			break;
-		}
-
-		g_free (update);
-	}
+	for (tem = processed; tem; tem = tem->next)
+		rhythmdb_query_model_free_pending_update (model, tem->data);
 		
 	g_list_free (processed);
 	return processed != NULL;
@@ -1083,6 +1154,15 @@ rhythmdb_query_model_row_drop_possible (RbTreeDragDest *drag_dest,
 	return model->priv->sort_func == NULL;
 }
 
+static gboolean
+rhythmdb_query_model_row_drop_position (RbTreeDragDest   *drag_dest,
+					GtkTreePath       *dest_path,
+					GList *targets,
+					GtkTreeViewDropPosition *pos)
+{
+	return TRUE;
+}
+
 static GtkTreeModelFlags
 rhythmdb_query_model_get_flags (GtkTreeModel *model)
 {
