@@ -45,7 +45,6 @@
 #include "rb-volume.h"
 #include "rb-bonobo-helpers.h"
 #include "rb-debug.h"
-#include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
 #include "rb-search-entry.h"
@@ -74,13 +73,13 @@ static void rb_library_source_get_property (GObject *object,
 			                  GValue *value,
 			                  GParamSpec *pspec);
 static void rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
-						 RBLibrarySource *source,
-						 const char *verbname);
+						RBShell *shell,
+						const char *verbname);
 static void rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
-						 RBLibrarySource *source,
+						 RBShell *shell,
 						 const char *verbname);
 static void rb_library_source_cmd_choose_album (BonoboUIComponent *component,
-						RBLibrarySource *source,
+						RBShell *shell,
 						const char *verbname);
 static void rb_library_source_handle_genre_selection (RBLibrarySource *libsource, GList *genres);
 static void rb_library_source_handle_artist_selection (RBLibrarySource *libsource, GList *artists);
@@ -118,6 +117,7 @@ static void rb_library_source_preferences_sync (RBLibrarySource *source);
 /* source methods */
 static const char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
+static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static GList *impl_get_extra_views (RBSource *source);
@@ -146,6 +146,7 @@ static GPtrArray * construct_query_from_selection (RBLibrarySource *source);
 #define CONF_STATE_LIBRARY_SORTING CONF_PREFIX "/ui/library/sorting"
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/library/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/library/show_browser"
+#define CONF_STATE_SEARCH_TEXT   CONF_PREFIX "/state/library/search_text"
 
 struct RBLibrarySourcePrivate
 {
@@ -187,6 +188,8 @@ struct RBLibrarySourcePrivate
 
 	GtkWidget *config_widget;
 	GSList *browser_views_group;
+
+	RhythmDBEntryType entry_type;
 };
 
 static BonoboUIVerb rb_library_source_verbs[] =
@@ -200,8 +203,9 @@ static BonoboUIVerb rb_library_source_verbs[] =
 enum
 {
 	PROP_0,
-	PROP_DB,
-	PROP_LIBRARY,
+	PROP_DB,	
+	PROP_ICON,
+	PROP_ENTRY_TYPE
 };
 
 static GObjectClass *parent_class = NULL;
@@ -251,6 +255,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	source_class->impl_get_status = impl_get_status;
 	source_class->impl_get_browser_key = impl_get_browser_key;
+	source_class->impl_get_search_key = impl_get_search_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -278,6 +283,23 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 							      "RhythmDB database",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_ICON,
+					 g_param_spec_object ("icon",
+							      "Icon",
+							      "Source Icon",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_uint ("entry-type",
+							    "Entry type",
+							    "Type of the entries which should be displayed by this source",
+							    0,
+							    G_MAXINT,
+							    RHYTHMDB_ENTRY_TYPE_SONG,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static void
@@ -287,29 +309,58 @@ update_browser_views_visibility (RBLibrarySource *source)
 	GtkWidget *genres = GTK_WIDGET (source->priv->genres);
 	GtkWidget *artists = GTK_WIDGET (source->priv->artists);
 	GtkWidget *albums = GTK_WIDGET (source->priv->albums);
+	BonoboUIComponent *component;
 
 	views = eel_gconf_get_integer (CONF_UI_LIBRARY_BROWSER_VIEWS);
+
+	g_object_get (G_OBJECT (source), "component", &component, NULL);
 	
 	switch (views)
 	{
 		case 0:
 			gtk_widget_hide (genres);
-			gtk_widget_show (artists);
-			gtk_widget_show (albums);
-			/* Since this is hidden now, reset the query */
 			rb_library_source_handle_genre_selection (source, NULL);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseGenre",
+						 FALSE);
+			gtk_widget_show (artists);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseArtist",
+						 TRUE);
+			gtk_widget_show (albums);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseAlbum",
+						 TRUE);
+			/* Since this is hidden now, reset the query */
 		break;
 		case 1:
 			gtk_widget_show (genres);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseGenre",
+						 TRUE);
 			gtk_widget_show (artists);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseArtist",
+						 TRUE);
 			gtk_widget_hide (albums);
 			rb_library_source_handle_artist_selection (source, NULL);
-
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseAlbum",
+						 FALSE);
 		break;
 		case 2:
 			gtk_widget_show (genres);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseGenre",
+						 TRUE);
 			gtk_widget_show (artists);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseArtist",
+						 TRUE);
 			gtk_widget_show (albums);
+			rb_bonobo_set_sensitive (component,
+						 "/commands" "/SLChooseAlbum",
+						 TRUE);
 		break;
 	}
 }
@@ -343,7 +394,6 @@ rb_library_source_ui_pref_changed (GConfClient *client,
 static void
 rb_library_source_init (RBLibrarySource *source)
 {
-	GtkWidget *dummy = gtk_tree_view_new ();
 	source->priv = g_new0 (RBLibrarySourcePrivate, 1);
 
 	/* Drag'n'Drop */
@@ -351,13 +401,6 @@ rb_library_source_init (RBLibrarySource *source)
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
-
-	source->priv->pixbuf = gtk_widget_render_icon (dummy,
-						       RB_STOCK_LIBRARY,
-						       GTK_ICON_SIZE_LARGE_TOOLBAR,
-						       NULL);
-
-	gtk_widget_destroy (dummy);
 }
 
 static void
@@ -408,6 +451,21 @@ rb_library_source_songs_show_popup_cb (RBEntryView *view,
 	gtk_object_sink (GTK_OBJECT (menu));
 }
 
+/* This function may seem a bit weird. Since we have 2 instances of objects 
+ * inheriting from RBLibrarySource, we can't install those ui verbs when
+ * creating a new instance of such an object. Attaching those verbs in the 
+ * shell would also be weird imo since this would add more code 
+ * dependant on the source type to the shell */
+void
+rb_library_source_class_add_verbs (RBShell *shell, 
+				   BonoboUIComponent *component)
+{
+	bonobo_ui_component_add_verb_list_with_data (component, 
+						     rb_library_source_verbs,
+						     shell);
+}
+
+
 static GObject *
 rb_library_source_constructor (GType type, guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
@@ -417,15 +475,13 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	GObjectClass *parent_class;  
 	BonoboUIComponent *component;
 
-	klass = RB_LIBRARY_SOURCE_CLASS (g_type_class_peek (type));
+	klass = RB_LIBRARY_SOURCE_CLASS (g_type_class_peek (RB_TYPE_LIBRARY_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	source = RB_LIBRARY_SOURCE (parent_class->constructor (type, n_construct_properties,
 							       construct_properties));
 
 	g_object_get (G_OBJECT (source), "component", &component, NULL);
-	bonobo_ui_component_add_verb_list_with_data (component, rb_library_source_verbs,
-						     source);
 
 	source->priv->paned = gtk_vpaned_new ();
 
@@ -495,8 +551,6 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	gtk_box_pack_start_defaults (GTK_BOX (source->priv->browser), GTK_WIDGET (source->priv->albums));
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned), source->priv->browser, FALSE, FALSE);
 
-	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
-
 	/* this gets emitted when the paned thingie is moved */
 	g_signal_connect (G_OBJECT (source->priv->songs),
 			  "size_allocate",
@@ -519,6 +573,7 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 				    (GConfClientNotifyFunc) rb_library_source_ui_pref_changed, source);
 	eel_gconf_notification_add (CONF_UI_LIBRARY_BROWSER_VIEWS,
 				    (GConfClientNotifyFunc) rb_library_source_browser_views_changed, source);
+	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
 	return G_OBJECT (source);
 }
 
@@ -536,6 +591,16 @@ rb_library_source_set_property (GObject *object,
 	case PROP_DB:
 		source->priv->db = g_value_get_object (value);
 		break;
+	case PROP_ICON:
+		if (source->priv->pixbuf) {
+			g_object_unref (G_OBJECT (source->priv->pixbuf));
+		}
+		source->priv->pixbuf = g_value_get_object (value);
+		g_object_ref (source->priv->pixbuf);
+		break;
+	case PROP_ENTRY_TYPE:
+		source->priv->entry_type = g_value_get_uint (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -555,6 +620,12 @@ rb_library_source_get_property (GObject *object,
 	case PROP_DB:
 		g_value_set_object (value, source->priv->db);
 		break;
+	case PROP_ICON:
+		g_value_set_object (value, source->priv->pixbuf);
+		break;
+	case PROP_ENTRY_TYPE:
+		g_value_set_uint (value, source->priv->entry_type);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -562,19 +633,32 @@ rb_library_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_library_source_new (RhythmDB *db, BonoboUIComponent *component)
+rb_library_source_new (RBShell *shell, RhythmDB *db, 
+		       BonoboUIComponent *component)
 {
 	RBSource *source;
+	GtkWidget *dummy = gtk_tree_view_new ();
+	GdkPixbuf *icon;
+
+	icon = gtk_widget_render_icon (dummy, RB_STOCK_LIBRARY,
+				       GTK_ICON_SIZE_LARGE_TOOLBAR,
+				       NULL);
+	gtk_widget_destroy (dummy);
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Library"),
-					  "component", component,
+					  "internal-name", "<library>",
+					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
 					  "db", db,
+					  "component", component,
+					  "icon", icon,
 					  NULL));
-
+	rb_shell_register_entry_type_for_source (shell, source, 
+						 RHYTHMDB_ENTRY_TYPE_SONG);
 	return source;
 }
 
+
 static gboolean
 string_list_equal (GList *a, GList *b)
 {
@@ -628,7 +712,6 @@ rb_library_source_gather_properties (RBLibrarySource *source,
 	GHashTable *selected_set;
 
 	rhythmdb_read_lock (source->priv->db);
-
 	selected_set = g_hash_table_new (g_str_hash, g_str_equal);
 	selected = rb_entry_view_get_selected_entries (source->priv->songs);
 	for (tem = selected; tem; tem = tem->next) {
@@ -650,13 +733,15 @@ rb_library_source_gather_properties (RBLibrarySource *source,
 
 static void
 rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
-				    RBLibrarySource *source,
+				    RBShell *shell,
 				    const char *verbname)
 {
 	GList *props;	
+	RBLibrarySource *source;
 
 	rb_debug ("choosing genre");
 
+	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_GENRE);
 	rb_property_view_set_selection (source->priv->genres, props);
 	g_list_free (props);
@@ -664,13 +749,15 @@ rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
 
 static void
 rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
-				     RBLibrarySource *source,
+				     RBShell *shell,
 				     const char *verbname)
 {
 	GList *props;	
+	RBLibrarySource *source;
 
 	rb_debug ("choosing artist");
 
+	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ARTIST);
 	rb_property_view_set_selection (source->priv->artists, props);
 	g_list_free (props);
@@ -678,13 +765,15 @@ rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
 
 static void
 rb_library_source_cmd_choose_album (BonoboUIComponent *component,
-				    RBLibrarySource *source,
+				    RBShell *shell,
 				    const char *verbname)
 {
 	GList *props;	
+	RBLibrarySource *source;
 
 	rb_debug ("choosing album");
 
+	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ALBUM);
 	rb_property_view_set_selection (source->priv->albums, props);
 	g_list_free (props);
@@ -817,6 +906,12 @@ impl_get_browser_key (RBSource *source)
 	return CONF_STATE_SHOW_BROWSER;
 }
 
+static const char *
+impl_get_search_key (RBSource *source)
+{
+	return CONF_STATE_SEARCH_TEXT;
+}
+
 static GdkPixbuf *
 impl_get_pixbuf (RBSource *asource)
 {
@@ -837,7 +932,10 @@ impl_search (RBSource *asource, const char *search_text)
 	    && !strcmp (search_text, source->priv->search_text))
 		return;
 
-	rb_debug ("doing search for \"%s\"", search_text);
+	if (search_text[0] == '\0')
+		search_text = NULL;
+
+	rb_debug ("doing search for \"%s\"", search_text ? search_text : "(NULL)");
 
 	g_free (source->priv->search_text);
 	source->priv->search_text = search_text != NULL ? g_utf8_casefold (search_text, -1) : NULL;
@@ -1230,12 +1328,15 @@ static GPtrArray *
 construct_query_from_selection (RBLibrarySource *source)
 {
 	GPtrArray *query;
+	RhythmDBEntryType entry_type;
 
+	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
-				      RHYTHMDB_ENTRY_TYPE_SONG,
+				      entry_type,
 				      RHYTHMDB_QUERY_END);
+
 	/* select where type="song"
 	 */
 
@@ -1341,8 +1442,9 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 			source->priv->search_text == NULL);
 	current_sorting_type = rb_entry_view_get_sorting_type (source->priv->songs);
 	sorting_matches = source->priv->cached_sorting_type
-		&& !strcmp (source->priv->cached_sorting_type, current_sorting_type);
+	&& !strcmp (source->priv->cached_sorting_type, current_sorting_type);
 	rb_debug ("current sorting: %s, match: %s", current_sorting_type, sorting_matches ? "TRUE" : "FALSE" );
+
 	if (is_all_query) {
 		if (sorting_matches) {
 			rb_debug ("cached query hit");
@@ -1471,8 +1573,4 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 		
 	rhythmdb_query_free (query);
 	g_object_unref (G_OBJECT (query_model));
-
-	rb_debug ("polling");
-	rb_entry_view_poll_model (source->priv->songs);
-	rb_debug ("done polling");
 }
