@@ -231,7 +231,7 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	GObjectClass *parent_class;  
 	GtkWidget *dummy = gtk_tree_view_new ();
 
-	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (type));
+	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	source = RB_PLAYLIST_SOURCE (parent_class->constructor (type, n_construct_properties,
@@ -432,10 +432,11 @@ rb_playlist_source_set_query (RBPlaylistSource *source,
 	g_assert (source->priv->automatic);
 
 	source->priv->model = query_model
-		= rhythmdb_query_model_new_empty (source->priv->db);
-	g_object_set (G_OBJECT (source->priv->model),
-		      "max-count", limit_count,
-		      "max-size", limit_mb, NULL);
+		= g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
+				"db", source->priv->db,
+				"max-count", limit_count,
+				"max-size", limit_mb, 
+				NULL);
 
 	model = GTK_TREE_MODEL (query_model);
 
@@ -787,10 +788,22 @@ rb_playlist_source_new_from_xml	(RhythmDB *db,
 	if (!strcmp (tmp, "automatic"))
 		source->priv->automatic = TRUE;
 	g_free (tmp);
+
 	tmp = xmlGetProp (node, "name");
 	g_object_set (G_OBJECT (source), "name", tmp, NULL);
 	g_free (tmp);
 
+	tmp = xmlGetProp (node, "internal-name");
+	if (!tmp) {
+		GTimeVal serial;
+		/* Hm.  Upgrades. */
+		g_get_current_time (&serial);
+		tmp = g_strdup_printf ("<playlist:%ld:%ld>",
+				       serial.tv_sec, serial.tv_usec);
+	}
+	g_object_set (G_OBJECT (source), "internal-name", tmp, NULL);
+	g_free (tmp);
+
 	if (source->priv->automatic) {
 		GPtrArray *query;
 		gchar *limit_str;
@@ -840,11 +853,14 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 {
 	xmlNodePtr node;
 	char *name;
+	char *internal_name;
 	GtkTreeIter iter;
 	
 	node = xmlNewChild (parent_node, NULL, "playlist", NULL);
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (G_OBJECT (source), "name", &name,
+		      "internal-name", &internal_name, NULL);
 	xmlSetProp (node, "name", name);
+	xmlSetProp (node, "internal-name", internal_name);
 	xmlSetProp (node, "type", source->priv->automatic ? "automatic" : "static");
 	
 	if (!source->priv->automatic) {
