@@ -790,6 +790,7 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	const char *b_str = NULL;
 	gint ret;
 
+	/* Sort by album name */
 	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
 	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
 
@@ -797,12 +798,20 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	if (ret != 0)
 		return ret;
 
+	/* Then by disc number, */
+	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_DISC_NUMBER);
+	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_DISC_NUMBER);
+	if (a_int != b_int)
+		return (a_int < b_int ? -1 : 1);
+
+	/* by track number */
 	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
 	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
 
 	if (a_int != b_int)
 		return (a_int < b_int ? -1 : 1);
 
+	/* And finally by title */
 	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_TITLE_SORT_KEY);
 	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_TITLE_SORT_KEY);
 
@@ -849,28 +858,7 @@ static gint
 rb_entry_view_track_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
-	gint a_int, b_int;
-	const char *a_str = NULL;
-	const char *b_str = NULL;	
-	gint ret;
-
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
-	if (ret != 0)
-		goto out;
-
-	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
-	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
-
-	if (a_int != b_int) {
-		ret = (a_int < b_int ? -1 : 1);
-		goto out;
-	}
-
-out:
-	return ret;
+	return rb_entry_view_album_sort_func (a, b, view);
 }
 
 
@@ -1441,7 +1429,7 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 	RBEntryView *view;
 	RBEntryViewClass *klass;
 	GObjectClass *parent_class;
-	klass = RB_ENTRY_VIEW_CLASS (g_type_class_peek (type));
+	klass = RB_ENTRY_VIEW_CLASS (g_type_class_peek (RB_TYPE_ENTRY_VIEW));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	view = RB_ENTRY_VIEW (parent_class->constructor (type, n_construct_properties,
@@ -1570,6 +1558,13 @@ rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 	g_value_unset (&value);
 
 	rb_entry_view_thaw (view);
+	
+	/* since the user changed the rating, stop auto-rating */
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, FALSE);
+	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_AUTO_RATE,
+				  &value);
+	g_value_unset (&value);
 }
 
 void
